package edu.ucf.cs.dml.simnets.measureme.devices;

/**
 * Non-Device-Activated simulator.  Subclasses of this class fire a data signal
 * immediately upon receiving a login but after they have fired an ACK signal.
 */
public abstract class NDADeviceSimulator extends OSCDeviceSimulator {
  protected String currentSimValue = getDefaultValue();

  public NDADeviceSimulator() {
    super();
  }

  /**
   * This method should return a default value for the device if the user
   * does not enter any value in simulator GUI.  Uses <code>compute</code>
   * method.
   * @return the default value
   */
  protected abstract String getDefaultValue();

  /**
   * Takes the user measurement (e.g. 12 inch foot length) and computes the
   * value the device simulator should output.  This value is the same value
   * that would come from the actual serial device.
   * @param userValue user value
   * @return device value
   */
  protected abstract String compute(Object userValue);

  /**
   * This method uses a customized <code>compute</code> method to fire a
   * simulator event when a gui event is received.
   * @param e gui event
   */
  public void guiEvent(SimulatorGUIEvent e) {
    // object at this point is a valid float of length in inches
    Float lengthInInches = (Float) e.getData();
    currentSimValue = compute(lengthInInches);
    // if there are listeners they will immediately get the value;
    //    if no listeners then they will get currentSimValue when
    //    they add themselves as listeners
    fireSimulatorEvent(new SimulatorEvent(this, currentSimValue));
  }

  /**
   * This method responds to a login with an ACK then sends the current value
   * held in the simulator.  Note that this method is called 3 times in order
   * to send the entire login code of 3 int's.
   * @param data 1/3 of device login code
   */
  public void write(int data) {
    if ((++loginCodeByteCounter % 3) == 0) {
      // start thread here to allow app to continue processing device signals
      new Thread(new Runnable() {
        public void run() {
          simGUI.appendStatus("Sim: Got login code...");
          loginCodeByteCounter = 0; // reset
          String ackString = new String(ACK_SIGNAL);
          simGUI.appendStatus("Sim: Sending ack signal...");
          // send ACK
          fireSimulatorEvent(new SimulatorEvent(NDADeviceSimulator.this,
                             ackString));
          // send current simulator value
          fireSimulatorEvent(new SimulatorEvent(NDADeviceSimulator.this,
                             currentSimValue));
        } // run
      }).start(); // Thread
    } // if
  }



} // NDADeviceSimulator