package edu.ucf.cs.dml.simnets.measureme.server;

import java.rmi.RemoteException;
import java.util.*;

import net.jini.core.lease.*;

public class AutomatedLeaseManager implements LeaseManager {
    private long granularity = 1000 * 60 * 5;  // five minutes
    private Hashtable leases = new Hashtable();

    public AutomatedLeaseManager() {
        RenewThread renewer = new RenewThread();
        renewer.start();
    }

    public AutomatedLeaseManager(long granularity) {
        this();
        this.granularity = granularity;
    }

    public void renewUntil(Lease lease, long stopTime,
        LeaseListener listener)
    {
        addLease(lease, stopTime, listener);
    }

    public void renewFor(Lease lease, long duration,
        LeaseListener listener)
    {
        addLease(lease, duration +
            System.currentTimeMillis(), listener);
    }

    private void addLease(Lease lease, long stopTime,
        LeaseListener listener)
    {
        if (stopTime == Lease.ANY || stopTime == Lease.FOREVER) {
            stopTime = Long.MAX_VALUE;
        }
        leases.put(lease, new LeaseInfo(stopTime, listener));
    }

    public void cancel(Lease lease)
        throws UnknownLeaseException, RemoteException
    {
        LeaseInfo info = (LeaseInfo)leases.get(lease);
        if (info != null) {
            if(leases.remove(lease)!=null)
             System.out.println("Lease removed successfully");
            lease.cancel();

            return;
        } else {
            throw new UnknownLeaseException();
        }
    }

    private class RenewThread extends Thread {
        public RenewThread() {
            super("Renew Manager");
        }

        public void run() {
            System.out.println("RenewManager Running");
            for (;;) {
                checkLeases();
                try {
                    Thread.sleep(granularity);
                } catch (InterruptedException e) {
                    return; // we've been asked to stop
                }
            }
        }
    }

        private class LeaseInfo {
            long stopTime;
            LeaseListener listener;

            public LeaseInfo(long stopTime, LeaseListener listener) {
                this.stopTime = stopTime;
                this.listener = listener;
            }
        }

    private void checkLeases() {
        LeaseMap leasesToRenew = null;
        long now = System.currentTimeMillis();

        Enumeration enum = leases.keys();
        while (enum.hasMoreElements()) {
            Lease lease = (Lease)enum.nextElement();
            LeaseInfo info = (LeaseInfo)leases.get(lease);

            // if lease's stop time has passed, remove it:
            if (info.stopTime < now) {
                if(leases.remove(lease)!=null)
                 System.out.println("Automatically removed lease");
                continue;
            }

            // if enough time left in lease, leave it alone:
            long expires = lease.getExpiration();
            if (expires > (now + (granularity * 2))) {
                continue;
            }

            // not enough time left, so add lease to leasesToRenew  map:
            if (leasesToRenew == null) {
                long duration = granularity * 2;
                leasesToRenew = lease.createLeaseMap(duration);
            } else {
                if (leasesToRenew.canContainKey(lease)) {
                    Long duration = new Long(granularity * 2);
                    leasesToRenew.put(lease, duration);
                } else {
                    info.listener.notify(
                        new LeaseRenewalEvent(this,
                            lease, info.stopTime,
                            new LeaseException("Not batchable")));
                    leases.remove(lease);
                }
            }
        }

        if (leasesToRenew != null) {
            try {
                leasesToRenew.renewAll();
            } catch (LeaseMapException e) {
                processExceptionMap(e.exceptionMap);
            } catch (RemoteException e) {
                ; // hope things improve next time around
            }
        }
    }

private void processExceptionMap(Map map) {
    if (map == null) {
        return;
    }

    Enumeration enum = leases.keys();
    while (enum.hasMoreElements()) {
        Lease lease = (Lease)enum.nextElement();
        if (map.containsKey(lease)) {
            LeaseInfo info = (LeaseInfo)leases.get(lease);
            info.listener.notify(
                new LeaseRenewalEvent(this,
                    lease, info.stopTime,
                    (LeaseException)map.get(lease)));
            leases.remove(lease);
        }
    }
}
}
