package edu.ucf.cs.dml.simnets.measureme.server;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2001</p>
 * <p>Company: </p>
 * @author Mathew B. Lowery
 * @version 1.0
 */

import edu.ucf.cs.dml.simnets.measureme.MeasureMeGlobal;
import net.jini.discovery.LookupDiscovery;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.Constants;
import net.jini.discovery.DiscoveryListener;
import java.io.IOException;
import net.jini.core.discovery.LookupLocator;
import java.net.MalformedURLException;
import java.util.Vector;
import net.jini.core.lookup.ServiceRegistrar;
import java.rmi.RMISecurityManager;

/**
 * A convenience class that encapsulates unicast and multicast discovery.  No
 * state is maintained (i.e. discovery is performed anew on every call).
 * @see JiniUtilException
 * @see ServiceSearchUtil
 * @see ServiceRegistrationUtil
 */
public class DiscoveryUtil {
  private static final boolean DEBUG = true;
  private static final long DEFAULT_TIMEOUT = 10 * 1000;  // 10 seconds
  private static final String[] DEFAULT_GROUPS = LookupDiscovery.ALL_GROUPS;
  private Vector registrars = null;
  private ServiceRegistrar registrar = null;
  private Object lock = new Object();

  public DiscoveryUtil() {
  } // constructor

  /**
   * Does a multicast discovery and returns at most one proxy belonging to any
   * groups.  <code>null</code> is returned if there is no match.
   * @return proxy to lookup service
   * @throws JiniUtilException
   */
  public ServiceRegistrar discoverOne() throws JiniUtilException {
    return discoverOne(DEFAULT_GROUPS, DEFAULT_TIMEOUT);
  } // discoverOne

  /**
   * Does a multicast discovery and returns at most one proxy belonging to the
   * given <code>groups</code>.  <code>null</code> is returned if there is no
   * match.
   * @param groups
   * @return proxy to lookup service
   * @throws JiniUtilException
   */
  public ServiceRegistrar discoverOne(String[] groups) throws JiniUtilException {
    return discoverOne(groups, DEFAULT_TIMEOUT);
  }

  /**
   * Does a multicast discovery and returns at most one proxy belonging to the
   * given <code>groups</code>.  Discovery stops after <code>timeout</code>.
   * <code>null</code> is returned if there is no match.
   * @param groups
   * @param timeout time to wait before returning (with or without a proxy)
   * @return proxy to lookup service
   * @throws JiniUtilException
   */
  public ServiceRegistrar discoverOne(String[] groups, long timeout) throws JiniUtilException {
    LookupDiscovery disco = null;
    try {
      disco = new LookupDiscovery(groups);
    }
    catch (IOException e) {
      if (DEBUG) e.printStackTrace();
      throw new JiniUtilException("problem during discovery");
    }
    registrars = new Vector();
    disco.addDiscoveryListener(new DiscoveryListener() {
      /**
      * Called when one or more lookup service registrars has been discovered.
      */
      public void discovered(DiscoveryEvent e) {
        ServiceRegistrar[] newregs = e.getRegistrars();
        if (newregs.length > 0) {
          registrar = newregs[0];  // use the first one
          synchronized(lock) {
            lock.notify();
          }
        }
      } // discovered

      /**
      * Called when one or more lookup service registrars has been discarded.
      */
      public void discarded(DiscoveryEvent e) {
        ServiceRegistrar[] newregs = e.getRegistrars();
        for (int i = 0; i < newregs.length; i++) {
          if (registrar == newregs[i]) {
            registrar = null;
          }
        }
      } // discarded
    });
    try {
      synchronized(lock) {
        lock.wait(timeout);
      }
    }
    catch (InterruptedException e) {
      if (DEBUG) e.printStackTrace();
    }
    disco.terminate();
    return registrar;
  }

  /**
   * Does a unicast discovery for lookup service running on <code>host</code>.
   * @param host machine running the lookup service
   * @return proxy to lookup service
   * @throws JiniUtilException
   */
  public ServiceRegistrar locate(String host) throws JiniUtilException {
    LookupLocator lookup = null;
    try {
      lookup = new LookupLocator("jini://"+host);
    }
    catch (MalformedURLException e) {
      if (DEBUG) e.printStackTrace();
      throw new JiniUtilException("invalid hostname: " + host);
    }
    ServiceRegistrar r = null;
    try {
      r = lookup.getRegistrar();
    }
    catch (ClassNotFoundException e) {
      if (DEBUG) e.printStackTrace();
      throw new JiniUtilException("problem during discovery");
    }
    catch (IOException e) {
      if (DEBUG) e.printStackTrace();
      throw new JiniUtilException("problem during discovery");
    }
    return r;
  } // locate

  /**
   * Does a multicast discovery of lookup services belonging to any groups
   * and returns proxies to lookup services that have responded within
   * a reasonable timeout.
   * @return collection of <code>ServiceRegistrar</code> objects
   * @throws JiniUtilException
   */
  public Vector discoverAll() throws JiniUtilException {
    return discoverAll(DEFAULT_GROUPS, DEFAULT_TIMEOUT);
  } // discoverAll

  /**
   * Does a multicast discovery of lookup services belonging to <code>groups</code>
   * and returns proxies to lookup services that have responded within a
   * reasonable timeout.
   * @param groups
   * @return collection of <code>ServiceRegistrar</code> objects
   * @throws JiniUtilException
   */
  public Vector discoverAll(String[] groups) throws JiniUtilException {
    return discoverAll(groups, DEFAULT_TIMEOUT);
  } // discoverAll

  /**
   * Performs multicast discovery of lookup services belonging to <code>groups</code>
   * and returns proxies to lookup services that have responded within
   * <code>timeout</code>.
   * @param groups
   * @param timeout time in milliseconds to wait
   * @return collection of <code>ServiceRegistrar</code> objects
   * @throws JiniUtilException
   */
  public Vector discoverAll(String[] groups, long timeout) throws JiniUtilException {
    LookupDiscovery disco = null;
    try {
      disco = new LookupDiscovery(groups);
    }
    catch (IOException e) {
      if (DEBUG) e.printStackTrace();
      throw new JiniUtilException("problem during discovery");
    }
    registrars = new Vector();
    disco.addDiscoveryListener(new DiscoveryListener() {
      /**
      * Called when one or more lookup service registrars has been discovered.
      */
      public void discovered(DiscoveryEvent e) {
        ServiceRegistrar[] newregs = e.getRegistrars();
        for (int i = 0; i < newregs.length; i++) {
          registrars.add(newregs[i]);
        }
      } // discovered

      /**
      * Called when one or more lookup service registrars has been discarded.
      */
      public void discarded(DiscoveryEvent e) {
        ServiceRegistrar[] newregs = e.getRegistrars();
        for (int i = 0; i < newregs.length; i++) {
          registrars.remove(newregs[i]);
        }
      } // discarded
    });
    try {
      Thread.currentThread().sleep(timeout);
    }
    catch (InterruptedException e) {
      if (DEBUG) e.printStackTrace();
    }
    disco.terminate();
    return registrars;
  } // discoverAll

  /**
   * Test the class here.  In order to run this class, the parameter
   * <code>-Djava.security.policy=<path to policy file></code> must exist.
   * This is because an instance of <code>ServiceRegistrar</code> is serialized
   * to this node and the actual implementation class is downloaded.  Permission
   * must be given for this class to be run.
   */
  public static void main(String[] args) throws Exception {
    System.setSecurityManager(new RMISecurityManager());
    DiscoveryUtil d = new DiscoveryUtil();
    ServiceRegistrar sr = null;
    System.out.println("Unicast Discovery");
    System.out.println("-----------------");
    sr = d.locate(MeasureMeGlobal.ipAddress);
    if (sr != null) {
      System.out.println("Got registrar: " + sr.toString());
    }
    System.out.println("");
    System.out.println("Multicast Discovery");
    System.out.println("-------------------");
    System.out.println("discoverOne");
    sr = d.discoverOne();
    if (sr != null) {
      System.out.println("Got registrar: " + sr.toString());
    }
    System.out.println("discoverAll");
    Vector v = d.discoverAll();
    for (int i = 0; i < v.size(); i++) {
      if (v.elementAt(i) != null) {
        System.out.println("Got registrar: " + v.elementAt(i).toString());
      }
    }
  } // main

} // DiscoveryUtil