
/**
 * Title:        MeasureMe Server Side<p>
 * Description:  Server side code.  <p>
 *  This Class runs periodic checks to determine which exhibits are still alive.
 *  If the exhibit has not checked in during some period of time, then it should be considered
 *  unavailable by other exibits.
 * Copyright:    Copyright (c) 1999<p>
 * Company:      DML/IST<p>
 * @author Dean Reed
 * @version
 */

// Edit M. Lowery 2002-Oct-1: removed recovery code; if an error occurs bail out
package edu.ucf.cs.dml.simnets.measureme.server;
import edu.ucf.cs.dml.simnets.measureme.dbase.*;
import net.jini.core.lease.Lease;
import net.jini.space.JavaSpace;
import net.jini.core.transaction.*;
import net.jini.core.transaction.server.TransactionManager;
import java.net.*;
import java.util.*;
import java.beans.*;


public class StationAvailableServer implements Runnable
{
  protected JavaSpace space=null;
  protected TransactionManager mgr=null;   //helps maintain state info across system
  protected long timeToCheck=150000;  //default check  every 2.5 minutes
  protected Thread myRunThread=null;
  protected boolean run=true;
  private transient PropertyChangeSupport propertyChangeListeners = new PropertyChangeSupport(this);
  private java.util.Vector lastActiveStations;//run thread sentinal

  /**
   * DEFAULT CONSTRUCTOR: check times are defaulted to 2.5 minutes
   */
  public StationAvailableServer()
  { this.space=MMSpaceAccessor.getSpace();
    this.mgr=MMTransactionManagerAccessor.getManager();
    this.myRunThread=new Thread(this);
    this.myRunThread.start();

  }

  /**
   *  CONSTRUCTOR: supply the time in which intermediate checks are made.
   */
  public StationAvailableServer(long timeToCheck)
  { this();
    this.timeToCheck=timeToCheck;
  }

  /**
   * CONSTRUCTOR: supply the javaspace services
   */
  public StationAvailableServer(JavaSpace space) {
//   if(space!=null)
     this.space=space;
//    else
//     this.recover();
    this.mgr=MMTransactionManagerAccessor.getManager();
    this.myRunThread=new Thread(this);
    this.myRunThread.start();
  }


  /**
   * CONSTRUCTOR: supply the javaspace services, and  supply the time in which intermediate checks are made
   */
  public StationAvailableServer(JavaSpace space,long timeToCheck)
  { this(space);
    this.timeToCheck=timeToCheck;
  }

  public StationAvailableServer(JavaSpace space,TransactionManager mgr,long timeToCheck)
  { this(space,timeToCheck);
    this.mgr=mgr;
  }

  public void run() {
    initialize(); //set up the java spaces with availbility info
    while (run) {
      try {
        myRunThread.sleep(timeToCheck);
        if (run) {
          this.runchecks(); //check which exhibits are up
        }
      }
      catch (InterruptedException e) {
        // ignore interruptions
        e.printStackTrace();
      }
    } // while
  } // run

  /**
   * PURPOSE: run the intermediate checks for availability
   */
  protected void runchecks() {
    Transaction.Created trc=null;
    Transaction txn=null;
    try {
      trc=TransactionFactory.create(this.mgr,60000); // a one minute lease
    }
    catch (Exception e) {
      // if we cannot create a transaction then exit
      e.printStackTrace();
      return;
    }
    try {
      ExhibitAvailableQueue template=new ExhibitAvailableQueue();
      template.stationsAvailableInfo=null;  //check javaspace for existing info
      txn=trc.transaction;     //get the transaction
      ExhibitAvailableQueue eaq=(ExhibitAvailableQueue)space.take(template,txn,Lease.FOREVER); //get rid of it
      if (!(eaq==null)) {
        eaq.performServerCheck();
        setLastActiveStations(eaq.getAvailableStations());
        space.write(eaq,txn,Lease.FOREVER);
        txn.commit();
      }
    }
    catch (Exception e2) {
      e2.printStackTrace();
      if (txn!=null) {
        try {
          txn.abort();
        }
        catch(Exception e3) {
          e3.printStackTrace();
        }
      }
      // If the exception is due to commit, then the txn will expire and the
      //   available queue will be put back unchanged.  This should not be a
      //   fatal error.  If the exception is due to a space.take, then
      //   availability status goes unchanged.  Again, should not be fatal.
    }
  } // runchecks

  protected boolean initialize() {
    ExhibitAvailableQueue template=new ExhibitAvailableQueue();
    template.stationsAvailableInfo=null;  //check javaspace for existing info
    try {
      if(space.take(template,null,30000)!=null) {
        //get rid of it
        System.out.println("Got rid of old queue");
      }
      else {
        System.out.println("Did not get rid of old que");
      }
    }
    catch (Exception e) {
      // if an exception occurs here, try to keep going
      e.printStackTrace();  //some problem with javaspace
    }
    try {
      System.out.println("waiting for Exhibit id info");
      QueryResults template2= new QueryResults();
      template2.setReplyToID("StationIDs");  //find the station ID's
      QueryResults result=null;
      ResultHolder rh=null;
      result=(QueryResults)space.read(template2,null,Lease.FOREVER);
      rh=result.getResults();
      Vector v=rh.getColumn(2); //all the station ids
      Vector v2=rh.getColumn(1);
      ExhibitAvailableQueue eaq=new ExhibitAvailableQueue();
      //create a new queue based on these results

      for (int i=0;i<v.size();i++) {
        eaq.addExhibitInfo(new ExhibitAvailableInfo((short)((Integer)v.elementAt(i)).intValue(),
                                                      (String)v2.elementAt(i)));
      }

      this.setLastActiveStations(eaq.getAvailableStations());
      System.out.println("adding new info to space");
      space.write(eaq,null,Lease.FOREVER);
    }
    catch (Exception e2) {
      e2.printStackTrace();
      return false;
    }
    return true;
  } // initialize

  /**
   * PURPOSE: Try to recover in the case of a javaspace problem.
   */
//  protected void recover() {
//    try {
//      this.mgr=MMTransactionManagerAccessor.getManager();
//      this.space=MMSpaceAccessor.getSpace();
//      initialize();
//    }
//    catch(Exception e) {
//      e.printStackTrace();
//    }
//  }

  /**
   * PURPOSE: Shuts down this server before the next checks are made
   */
  public synchronized void stop()
  { this.run=false;
    if(this.myRunThread!=null)
     this.myRunThread.interrupt();
  }




  public static void main(String[] args)
  {
    StationAvailableServer stationAvailableServer1 = new StationAvailableServer();
  }



  public synchronized void removePropertyChangeListener(PropertyChangeListener l)
  {
    propertyChangeListeners.removePropertyChangeListener(l);
  }

  public synchronized void addPropertyChangeListener(PropertyChangeListener l)
  {
    propertyChangeListeners.addPropertyChangeListener(l);
  }

  protected void setLastActiveStations(java.util.Vector newLastActiveStations)
  {
    java.util.Vector  oldLastActiveStations = lastActiveStations;
    lastActiveStations = newLastActiveStations;
    propertyChangeListeners.firePropertyChange("lastActiveStations", oldLastActiveStations, newLastActiveStations);
  }

  public java.util.Vector getLastActiveStations()
  {
    return lastActiveStations;
  }
} // StationAvailableServer